﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataPreprocessing {
  [Item("StringPreprocessingDataColumn", "")]
  [StorableClass]
  public sealed class StringPreprocessingDataColumn : PreprocessingDataColumn<string> {

    #region Constructor, Cloning & Persistence
    public StringPreprocessingDataColumn()
      : base() { }
    public StringPreprocessingDataColumn(string name, IEnumerable<string> values)
      : base(name, values) {
    }

    private StringPreprocessingDataColumn(StringPreprocessingDataColumn original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new StringPreprocessingDataColumn(this, cloner);
    }

    [StorableConstructor]
    private StringPreprocessingDataColumn(bool deserializing)
      : base(deserializing) { }
    #endregion

    public override bool IsValidValue(string value) {
      return !string.IsNullOrEmpty(value);
    }

    protected override string DefaultValue { get { return string.Empty; } }

    #region IStringConvertibleColumn
    public override bool Validate(string value, out string errorMessage) {
      errorMessage = string.Empty;
      return true;
    }
    public override string GetValue(int index) {
      return Values[index];
    }
    public override bool SetValue(string value, int index) {
      if (value != null) {
        Values[index] = value;
        return true;
      } else {
        return false;
      }
    }
    #endregion
  }
}